# coding=UTF-8

import sys
import re
import pysvg.parser
from optparse import OptionParser

parser = OptionParser("usage: %prog [options] foo.svg ")

parser.add_option ("-o", "--output-file", dest="output_file", type="str", default="code.c",
                  help="output file name, default: code.c")

(options, args) = parser.parse_args()

output_file = options.output_file

if len(args) != 1:
	print "Error: missing argument."
	sys.exit(1)

input_file = args[0]

SYM_EOF   = 0
SYM_NUM   = 1
SYM_m     = 2
SYM_M     = 3
SYM_c     = 4
SYM_C     = 5
SYM_l     = 6
SYM_L     = 7
SYM_z     = 8

parser_str = ""
parser_len = 0
parser_pos = 0
parser_num = 0.0
parser_sym = 0
parser_ch = ' '

# current point
parser_cpt = (0.0,0.0)

parser_path_open = False


def next_ch():
	global parser_pos, parser_len, parser_str, parser_ch
	if parser_pos >= parser_len:
		return 

	parser_ch = parser_str[parser_pos]
	parser_pos = parser_pos + 1

def is_whitespace(c):

	#print "is_whitespace: "+c

	if c <= ' ':
		#print "  yes"
		return True
	if c == ',':
		#print "  yes"
		return True
	#print "  no"
	return False

def is_number(c):
	if c >= '0' and c <= '9':
		return True
	if c == '-':
		return True
	if c == '.':
		return True
	if c == 'e':
		return True
	return False

def next_sym():
	global parser_ch, parser_pos, parser_len, parser_sym, parser_num

	if parser_pos >= parser_len:
		parser_sym = SYM_EOF
		return

	while is_whitespace (parser_ch):
		if parser_pos >= parser_len:
			parser_sym = SYM_EOF
			return
		next_ch()


	if parser_ch=='m':
		next_ch()
		parser_sym = SYM_m
	elif parser_ch=='M':
		next_ch()
		parser_sym = SYM_M
	elif parser_ch=='c':
		next_ch()
		parser_sym = SYM_c
	elif parser_ch=='C':
		next_ch()
		parser_sym = SYM_C
	elif parser_ch=='l':
		next_ch()
		parser_sym = SYM_l
	elif parser_ch=='L':
		next_ch()
		parser_sym = SYM_L
	elif parser_ch=='z':
		next_ch()
		parser_sym = SYM_z
	elif is_number (parser_ch):

		num_buf = ""
		while (parser_pos < parser_len) and is_number(parser_ch):
			num_buf = num_buf + parser_ch
			next_ch()

		if is_number(parser_ch):
			num_buf = num_buf + parser_ch

		parser_num = float(num_buf)
		print "   num_buf=" + num_buf
		print "   parser_num=" + str (parser_num)
		parser_sym = SYM_NUM
	else:
		raise Exception ( "next_sym: ERROR unexpected char '"+parser_ch+"'" )
		parser_sym = SYM_EOF

def parse_error(m):
	print m
	sys.exit()

def transform_coords (p, matrix) :

	print "transform_coords: p=%s, matrix=%s" % (str(p), str(matrix))

	x = matrix[0] * p[0] + matrix[2] * p[1] + matrix[4]
	y = matrix[1] * p[0] + matrix[3] * p[1] + matrix[5]
	
	print "transform_coords: %f/%f -> %f/%f" % (p[0], p[1], x, y)

	return (x,y)

def parse_transform (s):

	matrix = [0.0,0.0,0.0,0.0,0.0,0.0]

	m = re.match(r"matrix\(([-0-9.]+),([-0-9.]+),([-0-9.]+),([-0-9.]+),([-0-9.]+),([-0-9.]+)\)", s)

	if m:
		matrix[0] = float(m.group(1))
		matrix[1] = float(m.group(2))
		matrix[2] = float(m.group(3))
		matrix[3] = float(m.group(4))
		matrix[4] = float(m.group(5))
		matrix[5] = float(m.group(6))

		print "transform matrix %s parsed! result: %s" % (s, str(matrix))
	else:

		m = re.match (r"translate\(([-0-9.]+),([-0-9.]+)\)", s)
		if m:
			matrix[0] = 1.0
			matrix[1] = 0.0
			matrix[2] = 0.0
			matrix[3] = 1.0
			matrix[4] = float(m.group(1))
			matrix[5] = float(m.group(2))
			print "transform matrix %s parsed! result: %s" % (s, str(matrix))

		else:
			raise Exception ("transform matrix %s failed to parse!" % str)

	return matrix

def parse_coord_abs():
	if parser_sym != SYM_NUM:
		parse_error ("Coordinates expected")
	x = parser_num
	next_sym()
	if parser_sym != SYM_NUM:
		parse_error ("Coordinates expected")
	y = parser_num
	next_sym()
	return (x,y)

def parse_coord_rel():
	global parser_cpt

	if parser_sym != SYM_NUM:
		parse_error ("Coordinates expected")
	x = parser_num + parser_cpt[0]
	next_sym()
	if parser_sym != SYM_NUM:
		parse_error ("Coordinates expected")
	y = parser_num + parser_cpt[1]
	next_sym()
	return (x,y)


def parse_m(matrix, m):
	global parser_cpt, parser_path_open, outf
	
	p = parse_coord_rel()

	print "CODE: tvgfx_move_to (%f, %f);" % transform_coords(transform_coords(p, m), matrix)
	outf.write (" tvgfx_move_to (%f, %f);\n" % transform_coords(transform_coords(p, m), matrix))

	parser_cpt = p;
	parser_path_open = True

	while parser_sym == SYM_NUM:
		
		p = parse_coord_rel()

		print "CODE: tvgfx_line_to (%f, %f);" % transform_coords(transform_coords(p, m), matrix)
		outf.write (" tvgfx_line_to (%f, %f);\n" % transform_coords(transform_coords(p, m), matrix))

		parser_cpt = p;


def parse_M(matrix, m):
	global parser_cpt, parser_path_open, outf
	
	p = parse_coord_abs()

	print "CODE: tvgfx_move_to (%f, %f);" % transform_coords(transform_coords(p, m), matrix)
	outf.write (" tvgfx_move_to (%f, %f);\n" % transform_coords(transform_coords(p, m), matrix))

	parser_cpt = p;
	parser_path_open = True

	while parser_sym == SYM_NUM:
		
		p = parse_coord_abs()

		print "CODE: tvgfx_line_to (%f, %f);" % transform_coords(transform_coords(p, m), matrix)
		outf.write (" tvgfx_line_to (%f, %f);\n" % transform_coords(transform_coords(p, m), matrix))

		parser_cpt = p;


def parse_c():
	global parser_cpt, outf

	while parser_sym == SYM_NUM:
		
		p1 = parse_coord_rel()
		#parser_cpt = p1;
		p2 = parse_coord_rel()
		#parser_cpt = p2;
		p3 = parse_coord_rel()
		parser_cpt = p3;

		p1t = transform_coords(transform_coords(p1, m), matrix)
		p2t = transform_coords(transform_coords(p2, m), matrix)
		p3t = transform_coords(transform_coords(p3, m), matrix)

		print "CODE: tvgfx_cbezier_to (%f, %f, %f, %f, %f, %f);" % (p1t[0], p1t[1],
			p2t[0], p2t[1], p3t[0], p3t[1])

		outf.write (" tvgfx_cbezier_to (%f, %f, %f, %f, %f, %f);\n" % (p1t[0], p1t[1],
			p2t[0], p2t[1], p3t[0], p3t[1]))



def parse_C():
	global parser_cpt, outf

	while parser_sym == SYM_NUM:
		
		p1 = parse_coord_abs()
		p2 = parse_coord_abs()
		p3 = parse_coord_abs()

		p1t = transform_coords(transform_coords(p1, m), matrix)
		p2t = transform_coords(transform_coords(p2, m), matrix)
		p3t = transform_coords(transform_coords(p3, m), matrix)

		print "CODE: tvgfx_cbezier_to (%f, %f, %f, %f, %f, %f);" % (p1t[0], p1t[1],
			p2t[0], p2t[1], p3t[0], p3t[1])

		outf.write (" tvgfx_cbezier_to (%f, %f, %f, %f, %f, %f);\n" % (p1t[0], p1t[1],
			p2t[0], p2t[1], p3t[0], p3t[1]))

		parser_cpt = p3;


def parse_l():
	global parser_cpt, outf

	while parser_sym == SYM_NUM:
		
		p = parse_coord_rel()
		pt = transform_coords(transform_coords(p, m), matrix)

		print "CODE: tvgfx_line_to (%f, %f);" % (pt[0], pt[1])
		outf.write (" tvgfx_line_to (%f, %f);\n" % (pt[0], pt[1]))

		parser_cpt = p;



def parse_L():
	global parser_cpt, outf

	while parser_sym == SYM_NUM:
		
		p = parse_coord_abs()
		pt = transform_coords(transform_coords(p, m), matrix)

		print "CODE: tvgfx_line_to (%f, %f);" % (pt[0], pt[1])
		outf.write(" tvgfx_line_to (%f, %f);\n" % (pt[0], pt[1]))

		parser_cpt = p



def parse_d (d, matrix, m):

	global parser_str, parser_len, parser_sym, parser_num, parser_pos
	global parser_path_open, parser_cpt, outf

	parser_str = d
	parser_len = len(d)
	parser_pos = 0

	next_ch()
	next_sym()

	while parser_sym != SYM_EOF:

		print "parse_d: sym=%d" % parser_sym

		if parser_sym == SYM_m:
			next_sym()
			parse_m(matrix, m)
		elif parser_sym == SYM_M:
			next_sym()
			parse_M(matrix, m)
		elif parser_sym == SYM_c:
			next_sym()
			parse_c()
		elif parser_sym == SYM_C:
			next_sym()
			parse_C()
		elif parser_sym == SYM_l:
			next_sym()
			parse_l()
		elif parser_sym == SYM_L:
			next_sym()
			parse_L()
		elif parser_sym == SYM_z:
			next_sym()
			print "CODE: tvgfx_fill ();"
			outf.write (" tvgfx_fill ();\n")
			parser_path_open = False
		else:
			raise Exception("ERROR: unexpected symbol " + str (parser_sym))
	

	if parser_path_open:
		print "CODE: tvgfx_stroke();"
		outf.write (" tvgfx_stroke();\n");


svg = pysvg.parser.parse(input_file)
print svg.get_width(), svg.get_height()

outf = open(output_file,'w')

#
# fixed header
#

outf.write ("#include \"tvgfx/tvgfx.h\"\n")
outf.write ("#include \"tvfonts/allfonts.h\"\n")
outf.write ("\n")
outf.write ("void draw_shuttle(void) {\n")

print "svg: " + str(svg)

for element in svg._subElements:

	print "   element %s \n" % str(element)

	if isinstance (element, pysvg.structure.g):
		transform = element.get_transform()
		if transform:
			print "      G, transform: " + str (element.get_transform())
			matrix = parse_transform (element.get_transform())

			for element2 in element._subElements:

				print "         "+str(element2)
			
				if isinstance (element2, pysvg.shape.path):

					parser_cpt = (0.0,0.0)

					m = [1.0,0.0,0.0,1.0,0.0,0.0]
					trans = element2.get_transform()
					if trans:
						m = parse_transform(trans)


					d = element2.get_d()
					print "            d: "+d
					parse_d (d, matrix, m)

#		m = element.getNumElements():
#		for j in range(m):
#			element2 = element.getElementAt(j)

outf.write ("}\n")

